@@ -1,11 +1,12 @@
 # flake8: noqa: E402,F401
 from logging import getLogger
+from typing import Callable, Iterable, Dict
 
 logger = getLogger(__name__)
 
 # Version is defined in pyproject.toml.
 # It's copied here to make it easier for client code to check the installed version.
-__version__ = '0.7.5'
+__version__ = '0.8.1'
 
 
 def get_placeholder_class(original_exception: Exception = None):
@@ -31,10 +32,19 @@ def get_placeholder_class(original_exception: Exception = None):
     return Placeholder
 
 
+def get_valid_kwargs(func: Callable, kwargs: Dict, extras: Iterable[str] = None) -> Dict:
+    """Get the subset of non-None ``kwargs`` that are valid params for ``func``"""
+    params = list(signature(func).parameters)
+    params.extend(extras or [])
+    return {k: v for k, v in kwargs.items() if k in params and v is not None}
+
+
 try:
     from .backends import *
-    from .patcher import *
+    from .cache_control import DO_NOT_CACHE, CacheActions
+    from .cache_keys import create_key
     from .models import *
+    from .patcher import *
     from .serializers import *
     from .session import ALL_METHODS, CachedSession, CacheMixin
 # Log and ignore ImportErrors, if setup.py is invoked outside a virtualenv
