@@ -1,42 +1,61 @@
+"""Base classes for all cache backends.
+
+.. automodsumm:: requests_cache.backends.base
+   :classes-only:
+   :nosignatures:
+"""
 import pickle
-import warnings
 from abc import ABC
 from collections import UserDict
 from collections.abc import MutableMapping
 from datetime import datetime
 from logging import getLogger
-from typing import Iterable, Iterator, Tuple, Union
+from typing import Callable, Iterable, Iterator, Tuple, Union
 
 from ..cache_control import ExpirationTime
-from ..cache_keys import create_key, remove_ignored_params, remove_ignored_url_params, url_to_key
+from ..cache_keys import create_key, remove_ignored_params, remove_ignored_url_params
 from ..models import AnyRequest, AnyResponse, CachedResponse
 from ..serializers import init_serializer
 
 # Specific exceptions that may be raised during deserialization
 DESERIALIZE_ERRORS = (AttributeError, ImportError, TypeError, ValueError, pickle.PickleError)
 
+# Signature for user-provided callback
+KEY_FN = Callable[..., str]
+
 ResponseOrKey = Union[CachedResponse, str]
 logger = getLogger(__name__)
 
 
 class BaseCache:
-    """Base class for cache implementations, which can also be used as in-memory cache.
+    """Base class for cache backends. Can be used as a non-persistent, in-memory cache.
 
-    See :ref:`advanced_usage:custom backends` for details on creating your own implementation.
+    This manages higher-level cache operations, including:
+
+    * Cache expiration
+    * Generating cache keys
+    * Managing redirect history
+    * Convenience methods for general cache info
+
+    Lower-level storage operations are handled by :py:class:`.BaseStorage`.
+
+    To extend this with your own custom backend, see :ref:`custom-backends`.
     """
 
     def __init__(
         self,
         *args,
-        include_get_headers: bool = False,
+        match_headers: Union[Iterable[str], bool] = False,
         ignored_parameters: Iterable[str] = None,
+        key_fn: KEY_FN = None,
         **kwargs,
     ):
-        self.name: str = kwargs.get('cache_name', '')
-        self.redirects: BaseStorage = DictStorage()
         self.responses: BaseStorage = DictStorage()
-        self.include_get_headers = include_get_headers or kwargs.get('match_headers', False)
+        self.redirects: BaseStorage = DictStorage()
         self.ignored_parameters = ignored_parameters
+        self.key_fn = key_fn or create_key
+        self.name: str = kwargs.get('cache_name', '')
+        self.match_headers = match_headers or kwargs.pop('include_get_headers', False)
 
     @property
     def urls(self) -> Iterator[str]:
@@ -44,42 +63,18 @@ class BaseCache:
         for response in self.values():
             yield response.url
 
-    def save_response(self, response: AnyResponse, cache_key: str = None, expires: datetime = None):
-        """Save response to cache
-
-        Args:
-            cache_key: Cache key for this response; will otherwise be generated based on request
-            response: response to save
-            expire_after: Time in seconds until this cache item should expire
-        """
-        cache_key = cache_key or self.create_key(response.request)
-        cached_response = CachedResponse.from_response(response, cache_key=cache_key, expires=expires)
-        cached_response.request = remove_ignored_params(cached_response.request, self.ignored_parameters)
-        cached_response.url = remove_ignored_url_params(cached_response.url, self.ignored_parameters)
-        self.responses[cache_key] = cached_response
-
-    def save_redirect(self, request: AnyRequest, response_key: str):
-        """
-        Map a redirect request to a response. This makes it possible to associate many keys with a
-        single response.
-
-        Args:
-            request: Request object for redirect URL
-            response_key: Cache key which can be found in ``responses``
-        """
-        self.redirects[self.create_key(request)] = response_key
-
     def get_response(self, key: str, default=None) -> CachedResponse:
-        """Retrieves response for `key` if it's stored in cache, otherwise returns `default`
+        """Retrieve a response from the cache, if it exists
 
         Args:
-            key: Key of resource
-            default: Value to return if `key` is not in cache
+            key: Cache key for the response
+            default: Value to return if `key` is not in the cache
         """
         try:
             if key not in self.responses:
                 key = self.redirects[key]
             response = self.responses[key]
+            response.cache_key = key  # Set cache key here, since it's not serialized
             response.reset()  # In case response was in memory and content has already been read
             return response
         except KeyError:
@@ -89,6 +84,44 @@ class BaseCache:
             logger.debug(e, exc_info=True)
             return default
 
+    def save_response(self, response: AnyResponse, cache_key: str = None, expires: datetime = None):
+        """Save a response to the cache
+
+        Args:
+            cache_key: Cache key for this response; will otherwise be generated based on request
+            response: response to save
+            expire_after: Time in seconds until this cache item should expire
+        """
+        cache_key = cache_key or self.create_key(response.request)
+        cached_response = CachedResponse.from_response(response, expires=expires)
+        cached_response.url = remove_ignored_url_params(response.url, self.ignored_parameters)
+        cached_response.request = remove_ignored_params(cached_response.request, self.ignored_parameters)
+        self.responses[cache_key] = cached_response
+        for r in response.history:
+            self.redirects[self.create_key(r.request)] = cache_key
+
+    def bulk_delete(self, keys: Iterable[str]):
+        """Remove multiple responses and their associated redirects from the cache"""
+        self.responses.bulk_delete(keys)
+        # Remove any redirects that no longer point to an existing response
+        invalid_redirects = [k for k, v in self.redirects.items() if v not in self.responses]
+        self.redirects.bulk_delete(set(keys) | set(invalid_redirects))
+
+    def clear(self):
+        """Delete all items from the cache"""
+        logger.info('Clearing all items from the cache')
+        self.responses.clear()
+        self.redirects.clear()
+
+    def create_key(self, request: AnyRequest = None, **kwargs) -> str:
+        """Create a normalized cache key from a request object"""
+        return self.key_fn(
+            request=request,
+            ignored_parameters=self.ignored_parameters,
+            match_headers=self.match_headers,
+            **kwargs,
+        )
+
     def delete(self, key: str):
         """Delete a response or redirect from the cache, as well any associated redirect history"""
         # If it's a response key, first delete any associated redirect history
@@ -104,26 +137,30 @@ class BaseCache:
             except KeyError:
                 pass
 
-    def delete_url(self, url: str):
-        """Delete a cached response + redirects for ``GET <url>``"""
-        self.delete(url_to_key(url, self.ignored_parameters))
+    def delete_url(self, url: str, method: str = 'GET', **kwargs):
+        """Delete a cached response for the specified request"""
+        key = self.create_key(method=method, url=url, **kwargs)
+        self.delete(key)
 
-    def delete_urls(self, urls: Iterable[str]):
-        """Delete cached responses + redirects for multiple request URLs (``GET`` requests only)"""
-        self.bulk_delete([url_to_key(url, self.ignored_parameters) for url in urls])
+    def delete_urls(self, urls: Iterable[str], method: str = 'GET', **kwargs):
+        """Delete all cached responses for the specified requests"""
+        keys = [self.create_key(method=method, url=url, **kwargs) for url in urls]
+        self.bulk_delete(keys)
 
-    def bulk_delete(self, keys: Iterable[str]):
-        """Remove multiple responses and their associated redirects from the cache"""
-        self.responses.bulk_delete(keys)
-        # Remove any redirects that no longer point to an existing response
-        invalid_redirects = [k for k, v in self.redirects.items() if v not in self.responses]
-        self.redirects.bulk_delete(set(keys) | set(invalid_redirects))
+    def has_key(self, key: str) -> bool:
+        """Returns ``True`` if ``key`` is in the cache"""
+        return key in self.responses or key in self.redirects
 
-    def clear(self):
-        """Delete all items from the cache"""
-        logger.info('Clearing all items from the cache')
-        self.responses.clear()
-        self.redirects.clear()
+    def has_url(self, url: str, method: str = 'GET', **kwargs) -> bool:
+        """Returns ``True`` if the specified request is cached"""
+        key = self.create_key(method=method, url=url, **kwargs)
+        return self.has_key(key)  # noqa: W601
+
+    def keys(self, check_expiry=False) -> Iterator[str]:
+        """Get all cache keys for redirects and valid responses combined"""
+        yield from self.redirects.keys()
+        for key, _ in self._get_valid_responses(check_expiry=check_expiry):
+            yield key
 
     def remove_expired_responses(self, expire_after: ExpirationTime = None):
         """Remove expired and invalid responses from the cache, optionally with revalidation
@@ -153,40 +190,23 @@ class BaseCache:
             for key, response in keys_to_update.items():
                 self.responses[key] = response
 
-    def remove_old_entries(self, *args, **kwargs):
-        msg = 'BaseCache.remove_old_entries() is deprecated; please use CachedSession.remove_expired_responses()'
-        warnings.warn(DeprecationWarning(msg))
-        self.remove_expired_responses(*args, **kwargs)
-
-    def create_key(self, request: AnyRequest, **kwargs) -> str:
-        """Create a normalized cache key from a request object"""
-        return create_key(request, self.ignored_parameters, self.include_get_headers, **kwargs)  # type: ignore
-
-    def has_key(self, key: str) -> bool:
-        """Returns `True` if cache has `key`, `False` otherwise"""
-        return key in self.responses or key in self.redirects
-
-    def has_url(self, url: str) -> bool:
-        """Returns `True` if cache has `url`, `False` otherwise. Works only for GET request urls"""
-        return self.has_key(url_to_key(url, self.ignored_parameters))  # noqa: W601
-
-    def keys(self, check_expiry=False) -> Iterator[str]:
-        """Get all cache keys for redirects and valid responses combined"""
-        yield from self.redirects.keys()
-        for key, _ in self._get_valid_responses(check_expiry=check_expiry):
-            yield key
-
-    def values(self, check_expiry=False) -> Iterator[CachedResponse]:
-        """Get all valid response objects from the cache"""
-        for _, response in self._get_valid_responses(check_expiry=check_expiry):
-            yield response
-
     def response_count(self, check_expiry=False) -> int:
         """Get the number of responses in the cache, excluding invalid (unusable) responses.
         Can also optionally exclude expired responses.
         """
         return len(list(self.values(check_expiry=check_expiry)))
 
+    def update(self, other: 'BaseCache'):
+        """Update this cache with the contents of another cache"""
+        logger.debug(f'Copying {len(other.responses)} responses from {repr(other)} to {repr(self)}')
+        self.responses.update(other.responses)
+        self.redirects.update(other.redirects)
+
+    def values(self, check_expiry=False) -> Iterator[CachedResponse]:
+        """Get all valid response objects from the cache"""
+        for _, response in self._get_valid_responses(check_expiry=check_expiry):
+            yield response
+
     def _get_valid_responses(
         self, check_expiry=False, delete=False
     ) -> Iterator[Tuple[str, CachedResponse]]:
@@ -221,12 +241,21 @@ class BaseCache:
 
 
 class BaseStorage(MutableMapping, ABC):
-    """Base class for backend storage implementations
+    """Base class for backend storage implementations. This provides a common dictionary-like
+    interface for the underlying storage operations (create, read, update, delete). One
+    ``BaseStorage`` instance corresponds to a single table/hash/collection, or whatever the
+    backend-specific equivalent may be.
+
+    ``BaseStorage`` subclasses contain no behavior specific to ``requests`` or caching, which are
+    handled by :py:class:`.BaseCache`.
+
+    ``BaseStorage`` also contains a serializer module or instance (defaulting to :py:mod:`pickle`),
+    which determines how :py:class:`.CachedResponse` objects are saved internally. See
+    :ref:`serializers` for details.
 
     Args:
-        secret_key: Optional secret key used to sign cache items for added security
-        salt: Optional salt used to sign cache items
         serializer: Custom serializer that provides ``loads`` and ``dumps`` methods
+        kwargs: Additional serializer or backend-specific keyword arguments
     """
 
     def __init__(
@@ -238,9 +267,9 @@ class BaseStorage(MutableMapping, ABC):
         logger.debug(f'Initializing {type(self).__name__} with serializer: {self.serializer}')
 
     def bulk_delete(self, keys: Iterable[str]):
-        """Delete multiple keys from the cache. Does not raise errors for missing keys. This is a
-        basic version that subclasses should override with a more efficient backend-specific
-        version, if possible.
+        """Delete multiple keys from the cache, without raising errors for missing keys. This is a
+        naive implementation that subclasses should override with a more efficient backend-specific
+        implementation, if possible.
         """
         for k in keys:
             try:
@@ -253,4 +282,11 @@ class BaseStorage(MutableMapping, ABC):
 
 
 class DictStorage(UserDict, BaseStorage):
-    """A basic dict wrapper class for non-persistent storage"""
+    """A basic dict wrapper class for non-persistent, in-memory storage
+
+    .. note::
+        This is mostly a placeholder for when no other backends are available. For in-memory
+        caching, either :py:class:`.SQLiteCache` (with `use_memory=True`) or :py:class:`.RedisCache`
+        is recommended instead.
+
+    """
