@@ -1,3 +1,36 @@
+"""
+.. image::
+    ../_static/redis.png
+
+`Redis <https://redis.io>`_ is an in-memory data store with on-disk persistence. It offers a
+high-performace cache that scales exceptionally well, making it an ideal choice for larger
+applications.
+
+Persistence
+^^^^^^^^^^^
+Redis operates on data in memory, and by default also persists data to snapshots on disk. This is
+optimized for performance with a minor risk of data loss, which is usually the best configuration
+for a cache. If you need different behavior, the frequency and type of persistence can be customized
+or disabled entirely. See `Redis Persistence <https://redis.io/topics/persistence>`_ for details.
+
+Connection Options
+^^^^^^^^^^^^^^^^^^
+The Redis backend accepts any keyword arguments for :py:class:`redis.client.Redis`. These can be passed
+via :py:class:`.CachedSession`:
+
+    >>> session = CachedSession('http_cache', backend='redis', host='192.168.1.63', port=6379)
+
+Or via :py:class:`.RedisCache`:
+
+    >>> backend = RedisCache(host='192.168.1.63', port=6379)
+    >>> session = CachedSession('http_cache', backend=backend)
+
+API Reference
+^^^^^^^^^^^^^
+.. automodsumm:: requests_cache.backends.redis
+   :classes-only:
+   :nosignatures:
+"""
 from typing import Iterable
 
 from redis import Redis, StrictRedis
@@ -7,10 +40,10 @@ from . import BaseCache, BaseStorage, get_valid_kwargs
 
 
 class RedisCache(BaseCache):
-    """Redis cache backend.
+    """Redis cache backend
 
     Args:
-        namespace: redis namespace (default: ``'requests-cache'``)
+        namespace: Redis namespace
         connection: Redis connection instance to use instead of creating a new one
         kwargs: Additional keyword arguments for :py:class:`redis.client.Redis`
     """
@@ -24,24 +57,19 @@ class RedisCache(BaseCache):
 
 
 class RedisDict(BaseStorage):
-    """A dictionary-like interface for redis key-value store.
+    """A dictionary-like interface for Redis operations
 
-    Notes:
-        * In order to deal with how redis stores data/keys, all keys and data are pickled.
-        * The actual key name on the redis server will be ``namespace:collection_name``.
-
-    Args:
-        namespace: Redis namespace
-        collection_name: Name of the Redis hash map
-        connection: (optional) Redis connection instance to use instead of creating a new one
-        kwargs: Additional keyword arguments for :py:class:`redis.client.Redis`
+    **Notes:**
+        * In order to deal with how Redis stores data, all keys will be encoded and all values will
+          be serialized.
+        * The full hash name will be ``namespace:collection_name``
     """
 
     def __init__(self, namespace, collection_name='http_cache', connection=None, **kwargs):
         super().__init__(**kwargs)
         connection_kwargs = get_valid_kwargs(Redis, kwargs)
         self.connection = connection or StrictRedis(**connection_kwargs)
-        self._self_key = ':'.join([namespace, collection_name])
+        self._self_key = f'{namespace}:{collection_name}'
 
     def __getitem__(self, key):
         result = self.connection.hget(self._self_key, encode(key))
