@@ -1,51 +1,83 @@
-import hashlib
+"""Internal utilities for generating the cache keys that are used to match requests
+
+.. automodsumm:: requests_cache.cache_keys
+   :functions-only:
+   :nosignatures:
+"""
+from __future__ import annotations
+
 import json
+from hashlib import blake2b
 from operator import itemgetter
-from typing import Iterable, List, Mapping, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Dict, Iterable, List, Mapping, Optional, Tuple, Union
 from urllib.parse import parse_qsl, urlencode, urlparse, urlunparse
 
-from requests import PreparedRequest, Request, Session
+from requests import Request, Session
 from requests.models import CaseInsensitiveDict
 from requests.utils import default_headers
 from url_normalize import url_normalize
 
-DEFAULT_HEADERS = default_headers()
+from . import get_valid_kwargs
+
+if TYPE_CHECKING:
+    from .models import AnyRequest
+
+DEFAULT_REQUEST_HEADERS = default_headers()
+DEFAULT_EXCLUDE_HEADERS = {'Cache-Control', 'If-None-Match', 'If-Modified-Since'}
 RequestContent = Union[Mapping, str, bytes]
 
 
 def create_key(
-    request: PreparedRequest,
-    ignored_params: Iterable[str] = None,
-    include_get_headers: bool = False,
+    request: AnyRequest = None,
+    ignored_parameters: Iterable[str] = None,
+    match_headers: Union[Iterable[str], bool] = False,
     **kwargs,
 ) -> str:
-    """Create a normalized cache key from a request object"""
-    key = hashlib.sha256()
+    """Create a normalized cache key from either a request object or :py:class:`~requests.Request`
+    arguments
+    """
+    # Create a PreparedRequest, if needed
+    if not request:
+        request_kwargs = get_valid_kwargs(Request.__init__, kwargs)
+        request = Session().prepare_request(Request(**request_kwargs))
+    if TYPE_CHECKING:
+        assert request is not None
+
+    # Add method and relevant request settings
+    key = blake2b(digest_size=8)
     key.update(encode((request.method or '').upper()))
-    url = remove_ignored_url_params(request.url, ignored_params)
-    url = url_normalize(url)
-    key.update(encode(url))
     key.update(encode(kwargs.get('verify', True)))
 
-    body = remove_ignored_body_params(request, ignored_params)
+    # Add filtered/normalized URL + request params
+    url = remove_ignored_url_params(request.url, ignored_parameters)
+    key.update(encode(url_normalize(url)))
+
+    # Add filtered request body
+    body = remove_ignored_body_params(request, ignored_parameters)
     if body:
         key.update(body)
-    if include_get_headers and request.headers != DEFAULT_HEADERS:
-        for name, value in normalize_dict(request.headers).items():  # type: ignore
-            key.update(encode(f'{name}={value}'))
+
+    # Add filtered/normalized headers
+    headers = get_matched_headers(request.headers, ignored_parameters, match_headers)
+    for k, v in headers.items():
+        key.update(encode(f'{k}={v}'))
 
     return key.hexdigest()
 
 
-def remove_ignored_params(
-    request: PreparedRequest, ignored_params: Optional[Iterable[str]]
-) -> PreparedRequest:
-    if not ignored_params:
-        return request
-    request.headers = remove_ignored_headers(request.headers, ignored_params)
-    request.url = remove_ignored_url_params(request.url, ignored_params)
-    request.body = remove_ignored_body_params(request, ignored_params)
-    return request
+def get_matched_headers(
+    headers: CaseInsensitiveDict, ignored_parameters: Optional[Iterable[str]], match_headers
+) -> Dict:
+    """Get only the headers we should match against, given an optional include list and/or exclude
+    list. Also normalizes headers (sorted/lowercased keys).
+    """
+    if not match_headers:
+        return {}
+
+    included = set(match_headers if isinstance(match_headers, Iterable) else headers.keys())
+    included -= set(ignored_parameters or [])
+    included -= DEFAULT_EXCLUDE_HEADERS
+    return {k.lower(): headers[k] for k in sorted(included) if k in headers}
 
 
 def remove_ignored_headers(
@@ -61,6 +93,18 @@ def remove_ignored_headers(
     return headers
 
 
+def remove_ignored_params(
+    request: AnyRequest, ignored_parameters: Optional[Iterable[str]]
+) -> AnyRequest:
+    """Remove ignored parameters from request URL, body, and headers"""
+    if not ignored_parameters:
+        return request
+    request.headers = remove_ignored_headers(request.headers, ignored_parameters)
+    request.url = remove_ignored_url_params(request.url, ignored_parameters)
+    request.body = remove_ignored_body_params(request, ignored_parameters)
+    return request
+
+
 def remove_ignored_url_params(url: Optional[str], ignored_parameters: Optional[Iterable[str]]) -> str:
     """Remove any ignored request parameters from the URL"""
     if not ignored_parameters or not url:
@@ -81,9 +125,11 @@ def remove_ignored_url_params(url: Optional[str], ignored_parameters: Optional[I
 
 
 def remove_ignored_body_params(
-    request: PreparedRequest, ignored_parameters: Optional[Iterable[str]]
+    request: AnyRequest, ignored_parameters: Optional[Iterable[str]]
 ) -> bytes:
+    """Remove any ignored parameters from the request body"""
     original_body = request.body
+    filtered_body: Union[str, bytes] = b''
     content_type = request.headers.get('content-type')
     if not ignored_parameters or not original_body or not content_type:
         return encode(original_body)
@@ -96,13 +142,15 @@ def remove_ignored_body_params(
         body = _filter_params(sorted(body), ignored_parameters)
         filtered_body = json.dumps(body)
     else:
-        filtered_body = original_body  # type: ignore
+        filtered_body = original_body
 
     return encode(filtered_body)
 
 
-def _filter_params(data: List[Tuple[str, str]], ignored_params: Iterable[str]) -> List[Tuple[str, str]]:
-    return [(k, v) for k, v in data if k not in set(ignored_params)]
+def _filter_params(
+    data: List[Tuple[str, str]], ignored_parameters: Iterable[str]
+) -> List[Tuple[str, str]]:
+    return [(k, v) for k, v in data if k not in set(ignored_parameters)]
 
 
 def normalize_dict(
@@ -114,10 +162,6 @@ def normalize_dict(
         items: Request params, data, or json
         normalize_data: Also normalize stringified JSON
     """
-
-    def sort_dict(d):
-        return dict(sorted(d.items(), key=itemgetter(0)))
-
     if not items:
         return None
     if isinstance(items, Mapping):
@@ -134,9 +178,8 @@ def normalize_dict(
     return items
 
 
-def url_to_key(url: str, *args, **kwargs) -> str:
-    request = Session().prepare_request(Request('GET', url))
-    return create_key(request, *args, **kwargs)
+def sort_dict(d: Mapping) -> Dict:
+    return dict(sorted(d.items(), key=itemgetter(0)))
 
 
 def encode(value, encoding='utf-8') -> bytes:
@@ -146,6 +189,6 @@ def encode(value, encoding='utf-8') -> bytes:
 
 def decode(value, encoding='utf-8') -> str:
     """Decode a value from bytes, if hasn't already been.
-    Note: PreparedRequest.body is always encoded in utf-8.
+    Note: ``PreparedRequest.body`` is always encoded in utf-8.
     """
     return value.decode(encoding) if isinstance(value, bytes) else value
