@@ -1,4 +1,3 @@
-"""Classes to wrap cached response objects"""
 from datetime import datetime, timedelta, timezone
 from logging import getLogger
 from typing import TYPE_CHECKING, List, Optional, Tuple, Union
@@ -7,6 +6,7 @@ from attr import define, field
 from requests import PreparedRequest, Response
 from requests.cookies import RequestsCookieJar
 from requests.structures import CaseInsensitiveDict
+from urllib3.response import HTTPHeaderDict
 
 from ..cache_control import ExpirationTime, get_expiration_datetime
 from . import CachedHTTPResponse, CachedRequest
@@ -18,35 +18,32 @@ logger = getLogger(__name__)
 
 @define(auto_attribs=False, slots=False)
 class CachedResponse(Response):
-    """A serializable dataclass that emulates :py:class:`requests.Response`. Public attributes and
-    methods on CachedResponse objects will behave the same as those from the original response, but
-    with different internals optimized for serialization.
-
-    This means doing some pre- and post-initialization steps common to all serializers, such as
-    breaking nested objects down into their basic attributes and lazily re-initializing them, which
-    saves a bit of memory and deserialization steps when those objects aren't accessed.
+    """A class that emulates :py:class:`requests.Response`, with some additional optimizations
+    for serialization.
     """
 
     _content: bytes = field(default=None)
     _next: Optional[CachedRequest] = field(default=None)
-    url: str = field(default=None)
-    status_code: int = field(default=0)
-    cache_key: str = field(default=None)
+    cache_key: Optional[str] = None  # Not serialized; set by BaseCache.get_response()
     cookies: RequestsCookieJar = field(factory=RequestsCookieJar)
     created_at: datetime = field(factory=datetime.utcnow)
     elapsed: timedelta = field(factory=timedelta)
-    expires: Optional[datetime] = field(default=None)
     encoding: str = field(default=None)
-    headers: CaseInsensitiveDict = field(factory=dict)
+    expires: Optional[datetime] = field(default=None)
+    headers: CaseInsensitiveDict = field(factory=CaseInsensitiveDict)
     history: List['CachedResponse'] = field(factory=list)  # type: ignore
+    raw: CachedHTTPResponse = field(factory=CachedHTTPResponse, repr=False)
     reason: str = field(default=None)
     request: CachedRequest = field(factory=CachedRequest)  # type: ignore
-    raw: CachedHTTPResponse = field(factory=CachedHTTPResponse, repr=False)
+    status_code: int = field(default=0)
+    url: str = field(default=None)
 
     def __attrs_post_init__(self):
         """Re-initialize raw response body after deserialization"""
         if self.raw._body is None and self._content is not None:
             self.raw.reset(self._content)
+        if not self.raw.headers:
+            self.raw.headers = HTTPHeaderDict(self.headers)
 
     @classmethod
     def from_response(cls, original_response: Response, **kwargs):
@@ -76,12 +73,12 @@ class CachedResponse(Response):
         return obj
 
     @property
-    def _content_consumed(self):
-        """For compatibility with Response; will always be True for a cached response"""
+    def _content_consumed(self) -> bool:
+        """For compatibility with requests.Response; will always be True for a cached response"""
         return True
 
     @_content_consumed.setter
-    def _content_consumed(self, value):
+    def _content_consumed(self, value: bool):
         pass
 
     @property
@@ -112,12 +109,6 @@ class CachedResponse(Response):
         """Get the size of the response body in bytes"""
         return len(self.content) if self.content else 0
 
-    def __getstate__(self):
-        """Override pickling behavior in ``requests.Response.__getstate__``;
-        only required for python 3.6
-        """
-        return self.__dict__
-
     def __str__(self):
         return (
             f'request: {self.request}, response: {self.status_code} '
