@@ -1,4 +1,18 @@
-"""Main classes to add caching features to ``requests.Session``"""
+"""Main classes to add caching features to ``requests.Session``
+
+.. autosummary::
+   :nosignatures:
+
+   CachedSession
+   CacheMixin
+
+.. Explicitly show inherited method docs on CachedSession instead of CachedMixin
+.. autoclass:: requests_cache.session.CachedSession
+    :show-inheritance:
+    :inherited-members:
+
+.. autoclass:: requests_cache.session.CacheMixin
+"""
 from contextlib import contextmanager
 from logging import getLogger
 from threading import RLock
@@ -9,15 +23,16 @@ from requests import Session as OriginalSession
 from requests.hooks import dispatch_hook
 from urllib3 import filepost
 
-from .backends import BackendSpecifier, get_valid_kwargs, init_backend
+from . import get_valid_kwargs
+from .backends import BackendSpecifier, init_backend
 from .cache_control import CacheActions, ExpirationTime
 from .cache_keys import normalize_dict
-from .models import AnyResponse, set_response_defaults
+from .models import AnyResponse, CachedResponse, set_response_defaults
 
 ALL_METHODS = ['GET', 'HEAD', 'OPTIONS', 'POST', 'PUT', 'PATCH', 'DELETE']
+FILTER_FN = Callable[[AnyResponse], bool]
 
 logger = getLogger(__name__)
-# MIXIN_BASE: Type = OriginalSession if TYPE_CHECKING else object
 if TYPE_CHECKING:
     MIXIN_BASE = OriginalSession
 else:
@@ -26,7 +41,7 @@ else:
 
 class CacheMixin(MIXIN_BASE):
     """Mixin class that extends :py:class:`requests.Session` with caching features.
-    See :py:class:`.CachedSession` for usage information.
+    See :py:class:`.CachedSession` for usage details.
     """
 
     def __init__(
@@ -35,28 +50,28 @@ class CacheMixin(MIXIN_BASE):
         backend: BackendSpecifier = None,
         expire_after: ExpirationTime = -1,
         urls_expire_after: Dict[str, ExpirationTime] = None,
+        cache_control: bool = False,
         allowable_codes: Iterable[int] = (200,),
         allowable_methods: Iterable[str] = ('GET', 'HEAD'),
-        filter_fn: Callable = None,
-        old_data_on_error: bool = False,
-        cache_control: bool = False,
+        filter_fn: FILTER_FN = None,
+        stale_if_error: bool = False,
         **kwargs,
     ):
-        self.cache = init_backend(backend, cache_name, **kwargs)
+        self.cache = init_backend(cache_name, backend, **kwargs)
         self.allowable_codes = allowable_codes
         self.allowable_methods = allowable_methods
         self.expire_after = expire_after
         self.urls_expire_after = urls_expire_after
-        self.filter_fn = filter_fn or (lambda r: True)
-        self.old_data_on_error = old_data_on_error or kwargs.get('stale_if_error', False)
         self.cache_control = cache_control
+        self.filter_fn = filter_fn or (lambda r: True)
+        self.stale_if_error = stale_if_error or kwargs.pop('old_data_on_error', False)
 
         self.cache.name = cache_name  # Set to handle backend=<instance>
         self._request_expire_after: ExpirationTime = None
         self._disabled = False
         self._lock = RLock()
 
-        # If the superclass is custom Session, pass along valid kwargs (if any)
+        # If the superclass is custom Session, pass along any valid kwargs
         session_kwargs = get_valid_kwargs(super().__init__, kwargs)
         super().__init__(**session_kwargs)  # type: ignore
 
@@ -111,7 +126,7 @@ class CacheMixin(MIXIN_BASE):
         """Send a prepared request, with caching. See :py:meth:`.request` for notes on behavior."""
         # Determine which actions to take based on request info, headers, and cache settings
         cache_key = self.cache.create_key(request, **kwargs)
-        actions = CacheActions(
+        actions = CacheActions.from_request(
             cache_key=cache_key,
             request=request,
             request_expire_after=self._request_expire_after,
@@ -122,72 +137,84 @@ class CacheMixin(MIXIN_BASE):
         )
 
         # Attempt to fetch a cached response
-        response: Optional[AnyResponse] = None
+        cached_response: Optional[CachedResponse] = None
         if not (self._disabled or actions.skip_read):
-            response = self.cache.get_response(cache_key)
-        is_expired = getattr(response, 'is_expired', False)
+            cached_response = self.cache.get_response(cache_key)
+            actions.update_from_cached_response(cached_response)
+        is_expired = getattr(cached_response, 'is_expired', False)
 
-        # If the cache is disabled, doesn't have the response, or it's expired, then fetch a new one
-        if response is None:
+        # If the response is expired, missing, or the cache is disabled, then fetch a new response
+        if cached_response is None:
             response = self._send_and_cache(request, actions, **kwargs)
-        elif is_expired and self.old_data_on_error:
-            response = self._resend_and_ignore(request, actions, **kwargs) or response
+        elif is_expired and self.stale_if_error:
+            response = self._resend_and_ignore(request, actions, cached_response, **kwargs)
         elif is_expired:
-            response = self._resend(request, actions, **kwargs)
+            response = self._resend(request, actions, cached_response, **kwargs)
+        else:
+            response = cached_response
 
-        # Dispatch any hooks here, because they are removed before pickling
-        response = dispatch_hook('response', request.hooks, response, **kwargs)
-        if TYPE_CHECKING:
-            assert response is not None
-
-        # If the request has been filtered out, delete previously cached response if it exists
+        # If the request has been filtered out, delete the previously cached response if it exists
         if not self.filter_fn(response):
             logger.debug(f'Deleting filtered response for URL: {response.url}')
             self.cache.delete(cache_key)
             return response
 
-        # Cache redirect history
-        for r in response.history:
-            self.cache.save_redirect(r.request, cache_key)
-        return response
+        # Dispatch any hooks here, because they are removed before pickling
+        return dispatch_hook('response', request.hooks, response, **kwargs)
 
-    def _send_and_cache(self, request: PreparedRequest, actions: CacheActions, **kwargs):
-        """Send the request and cache the response, unless disabled by settings or headers"""
+    def _send_and_cache(
+        self,
+        request: PreparedRequest,
+        actions: CacheActions,
+        cached_response: CachedResponse = None,
+        **kwargs,
+    ) -> AnyResponse:
+        """Send the request and cache the response, unless disabled by settings or headers.
+
+        If applicable, also add request headers to check if the remote resource has been modified.
+        If we get a 304 Not Modified response, return the expired cache item.
+        """
+        request.headers.update(actions.request_headers)
         response = super().send(request, **kwargs)
         actions.update_from_response(response)
 
         if self._is_cacheable(response, actions):
             self.cache.save_response(response, actions.cache_key, actions.expires)
+        elif cached_response and response.status_code == 304:
+            logger.debug(f'Response for URL {request.url} has not been modified; using cached response')
+            return cached_response
         else:
             logger.debug(f'Skipping cache write for URL: {request.url}')
         return set_response_defaults(response, actions.cache_key)
 
-    def _resend(self, request: PreparedRequest, actions: CacheActions, **kwargs) -> AnyResponse:
+    def _resend(
+        self, request: PreparedRequest, actions: CacheActions, cached_response: CachedResponse, **kwargs
+    ) -> AnyResponse:
         """Attempt to resend the request and cache the new response. If the request fails, delete
         the expired cache item.
         """
         logger.debug('Expired response; attempting to re-send request')
         try:
-            return self._send_and_cache(request, actions, **kwargs)
+            return self._send_and_cache(request, actions, cached_response, **kwargs)
         except Exception:
             self.cache.delete(actions.cache_key)
             raise
 
     def _resend_and_ignore(
-        self, request: PreparedRequest, actions: CacheActions, **kwargs
-    ) -> Optional[AnyResponse]:
-        """Attempt to send the request and cache the new response. If there are any errors, ignore
-        them and and return ``None``.
+        self, request: PreparedRequest, actions: CacheActions, cached_response: CachedResponse, **kwargs
+    ) -> AnyResponse:
+        """Attempt to resend the request and cache the new response. If there are any errors, ignore
+        them and and return the expired cache item.
         """
         # Attempt to send the request and cache the new response
         logger.debug('Expired response; attempting to re-send request')
         try:
-            response = self._send_and_cache(request, actions, **kwargs)
+            response = self._send_and_cache(request, actions, cached_response, **kwargs)
             response.raise_for_status()
             return response
-        except Exception as e:
-            logger.warning('Request failed; using stale cache data: %s', e)
-            return None
+        except Exception:
+            logger.warning(f'Request for URL {request.url} failed; using cached response', exc_info=True)
+            return cached_response
 
     def _is_cacheable(self, response: Response, actions: CacheActions) -> bool:
         """Perform all checks needed to determine if the given response should be saved to the cache"""
@@ -249,7 +276,7 @@ class CacheMixin(MIXIN_BASE):
             'urls_expire_after',
             'allowable_codes',
             'allowable_methods',
-            'old_data_on_error',
+            'stale_if_error',
             'cache_control',
         ]
         attr_strs = [f'{k}={repr(getattr(self, k))}' for k in repr_attrs]
@@ -257,7 +284,7 @@ class CacheMixin(MIXIN_BASE):
 
 
 class CachedSession(CacheMixin, OriginalSession):
-    """Class that extends :py:class:`requests.Session` with caching features.
+    """Session class that extends :py:class:`requests.Session` with caching features.
 
     See individual :py:mod:`backend classes <requests_cache.backends>` for additional backend-specific arguments.
     Also see :ref:`user-guide` for more details and examples on how the following arguments
@@ -265,26 +292,30 @@ class CachedSession(CacheMixin, OriginalSession):
 
     Args:
         cache_name: Cache prefix or namespace, depending on backend
-        backend: Cache backend name, class, or instance; name may be one of
-            ``['sqlite', 'mongodb', 'gridfs', 'redis', 'dynamodb', 'memory']``.
+        backend: Cache backend name or instance; name may be one of
+            ``['sqlite', 'filesystem', 'mongodb', 'gridfs', 'redis', 'dynamodb', 'memory']``
+        serializer: Serializer name or instance; name may be one of
+            ``['pickle', 'json', 'yaml', 'bson']``.
         expire_after: Time after which cached items will expire
         urls_expire_after: Expiration times to apply for different URL patterns
-        allowable_codes: Only cache responses with one of these codes
+        cache_control: Use Cache-Control headers to set expiration
+        allowable_codes: Only cache responses with one of these status codes
         allowable_methods: Cache only responses for one of these HTTP methods
-        include_get_headers: Make request headers part of the cache key
-        ignored_parameters: List of request parameters to be excluded from the cache key
-        filter_fn: function that takes a :py:class:`aiohttp.ClientResponse` object and
-            returns a boolean indicating whether or not that response should be cached. Will be
-            applied to both new and previously cached responses.
-        old_data_on_error: Return stale cache data if a new request raises an exception
-        cache_control: Use Cache-Control request and response headers
+        match_headers: Match request headers when reading from the cache; may be either a boolean
+            or a list of specific headers to match
+        ignored_parameters: List of request parameters to not match against, and exclude from the cache
+        filter_fn: Function that takes a :py:class:`~requests.Response` object and returns a boolean
+            indicating whether or not that response should be cached. Will be applied to both new
+            and previously cached responses.
+        key_fn: Function for generating custom cache keys based on request info
+        stale_if_error: Return stale cache data if a new request raises an exception
     """
 
 
 @contextmanager
 def patch_form_boundary(**request_kwargs):
-    """This patches the form boundary used to separate multipart uploads. Requests does not
-    provide a way to pass a custom boundary to urllib3, so this just monkey-patches it instead.
+    """Patch the form boundary used to separate multipart uploads. ``requests`` does not provide a
+    way to pass a custom boundary to urllib3, so this just monkey-patches it instead.
     """
     if request_kwargs.get('files'):
         original_boundary = filepost.choose_boundary
