@@ -9,11 +9,11 @@ Note: The protocol ``http(s)+mock://`` helps :py:class:`requests_mock.Adapter` p
 https://requests-mock.readthedocs.io/en/latest/adapter.html
 """
 import os
-from datetime import datetime
+from datetime import datetime, timedelta
 from functools import wraps
 from logging import basicConfig, getLogger
 from os.path import abspath, dirname, join
-from tempfile import NamedTemporaryFile
+from uuid import uuid4
 
 import pytest
 import requests
@@ -50,6 +50,9 @@ HTTPBIN_FORMATS = [
 
 HTTPDATE_STR = 'Fri, 16 APR 2021 21:13:00 GMT'
 HTTPDATE_DATETIME = datetime(2021, 4, 16, 21, 13)
+EXPIRED_DT = datetime.now() - timedelta(1)
+ETAG = '"644b5b0155e6404a9cc4bd9d8b1ae730"'
+LAST_MODIFIED = 'Thu, 05 Jul 2012 15:31:30 GMT'
 
 MOCKED_URL = 'http+mock://requests-cache.com/text'
 MOCKED_URL_HTTPS = 'https+mock://requests-cache.com/text'
@@ -104,7 +107,7 @@ def httpbin_wrapper(httpbin):
 
 
 @pytest.fixture(scope='function')
-def mock_session() -> CachedSession:
+def mock_session(tempfile_session) -> CachedSession:
     """Fixture for combining requests-cache with requests-mock. This will behave the same as a
     CachedSession, except it will make mock requests for ``mock://`` URLs, if it hasn't been cached
     already.
@@ -116,46 +119,36 @@ def mock_session() -> CachedSession:
     This uses a temporary SQLite db stored in ``/tmp``, which will be removed after the fixture has
     exited.
     """
-    with NamedTemporaryFile(suffix='.db') as temp:
-        session = CachedSession(
-            db_path=temp.name,
-            backend='sqlite',
-            allowable_methods=ALL_METHODS,
-        )
-        yield mount_mock_adapter(session)
+    yield mount_mock_adapter(tempfile_session)
 
 
 @pytest.fixture(scope='function')
-def tempfile_session() -> CachedSession:
+def tempfile_session(tempfile_path) -> CachedSession:
     """Get a CachedSession using a temporary SQLite db"""
-    with NamedTemporaryFile(suffix='.db') as temp:
-        session = CachedSession(
-            cache_name=temp.name,
-            backend='sqlite',
-            allowable_methods=ALL_METHODS,
-        )
-        yield session
+    yield CachedSession(
+        cache_name=tempfile_path,
+        backend='sqlite',
+        allowable_methods=ALL_METHODS,
+    )
 
 
 @pytest.fixture(scope='function')
-def tempfile_path() -> CachedSession:
-    """Get a tempfile path that will be deleted after the test finishes"""
-    with NamedTemporaryFile(suffix='.db') as temp:
-        yield temp.name
+def tempfile_path(tmpdir) -> str:
+    """Get a unique tempfile path"""
+    yield str(tmpdir / f'{uuid4()}.db')
 
 
 @pytest.fixture(scope='function')
-def installed_session() -> CachedSession:
+def installed_session(tempfile_path) -> CachedSession:
     """Get a CachedSession using a temporary SQLite db, with global patching.
     Installs cache before test and uninstalls after.
     """
-    with NamedTemporaryFile(suffix='.db') as temp:
-        requests_cache.install_cache(
-            cache_name=temp.name,
-            backend='sqlite',
-            allowable_methods=ALL_METHODS,
-        )
-        yield requests.Session()
+    requests_cache.install_cache(
+        cache_name=tempfile_path,
+        backend='sqlite',
+        allowable_methods=ALL_METHODS,
+    )
+    yield requests.Session()
     requests_cache.uninstall_cache()
 
 
@@ -213,7 +206,7 @@ def get_mock_adapter() -> Adapter:
     return adapter
 
 
-def fail_if_no_connection(func) -> bool:
+def fail_if_no_connection(connect_timeout: float = 1.0) -> bool:
     """Decorator for testing a backend connection. This will intentionally cause a test failure if
     the wrapped function doesn't have dependencies installed, doesn't connect after a short timeout,
     or raises any exceptions.
@@ -222,15 +215,18 @@ def fail_if_no_connection(func) -> bool:
     extended period of time.
     """
 
-    @wraps(func)
-    def wrapper(*args, **kwargs):
-        try:
-            timeout(1.0, use_signals=False)(func)(*args, **kwargs)
-        except Exception as e:
-            logger.error(e)
-            pytest.fail('Could not connect to backend')
+    def decorator(func):
+        @wraps(func)
+        def wrapper(*args, **kwargs):
+            try:
+                timeout(connect_timeout, use_signals=False)(func)(*args, **kwargs)
+            except Exception as e:
+                logger.error(e)
+                pytest.fail('Could not connect to backend')
 
-    return wrapper
+        return wrapper
+
+    return decorator
 
 
 def assert_delta_approx_equal(dt1: datetime, dt2: datetime, target_delta, threshold_seconds=2):
