@@ -9,7 +9,7 @@ from tests.integration.base_storage_test import BaseStorageTest
 
 
 @pytest.fixture(scope='module', autouse=True)
-@fail_if_no_connection
+@fail_if_no_connection()
 def ensure_connection():
     """Fail all tests in this module if Redis is not running"""
     from redis import Redis
