@@ -11,17 +11,16 @@ import pytest
 import requests
 from requests.structures import CaseInsensitiveDict
 
-from requests_cache import ALL_METHODS, CachedSession
+from requests_cache import ALL_METHODS, CachedResponse, CachedSession
 from requests_cache.backends import (
     BACKEND_CLASSES,
     BaseCache,
-    DbDict,
-    DbPickleDict,
+    SQLiteDict,
+    SQLitePickleDict,
     get_placeholder_class,
 )
 from requests_cache.backends.base import DESERIALIZE_ERRORS
-from requests_cache.cache_keys import url_to_key
-from requests_cache.models import CachedResponse
+from requests_cache.cache_keys import create_key
 from tests.conftest import (
     MOCKED_URL,
     MOCKED_URL_404,
@@ -61,17 +60,6 @@ def test_init_backend_class():
     assert isinstance(session.cache, MyCache)
 
 
-def test_import_compat():
-    """Just make sure that we can still import from requests_cache.core"""
-    with pytest.deprecated_call():
-        from requests_cache.core import CachedSession, install_cache  # noqa: F401
-
-
-def test_method_compat(mock_session):
-    with pytest.deprecated_call():
-        mock_session.cache.remove_old_entries()
-
-
 @pytest.mark.parametrize('method', ALL_METHODS)
 @pytest.mark.parametrize('field', ['params', 'data', 'json'])
 def test_all_methods(field, method, mock_session):
@@ -84,7 +72,7 @@ def test_all_methods(field, method, mock_session):
 
 
 @pytest.mark.parametrize('method', ALL_METHODS)
-@pytest.mark.parametrize('field', ['params', 'data', 'json'])
+@pytest.mark.parametrize('field', ['params', 'headers', 'data', 'json'])
 def test_all_methods__ignored_parameters__not_matched(field, method, mock_session):
     """Test all relevant combinations of methods and data fields. Requests with different request
     params, data, or json should not be cached under different keys based on an ignored param.
@@ -168,7 +156,7 @@ def test_urls(mock_session):
 def test_urls__with_invalid_response(mock_session):
     responses = [mock_session.get(url) for url in [MOCKED_URL, MOCKED_URL_JSON, MOCKED_URL_HTTPS]]
     responses[2] = AttributeError
-    with patch.object(DbPickleDict, '__getitem__', side_effect=responses):
+    with patch.object(SQLitePickleDict, '__getitem__', side_effect=responses):
         expected_urls = [MOCKED_URL, MOCKED_URL_JSON]
         assert set(mock_session.cache.urls) == set(expected_urls)
 
@@ -184,6 +172,17 @@ def test_keys(mock_session):
     assert set(mock_session.cache.keys()) == all_keys
 
 
+def test_update(mock_session):
+    src_cache = BaseCache()
+    for i in range(20):
+        src_cache.responses[f'key_{i}'] = f'value_{i}'
+        src_cache.redirects[f'key_{i}'] = f'value_{i}'
+
+    mock_session.cache.update(src_cache)
+    assert len(mock_session.cache.responses) == 20
+    assert len(mock_session.cache.redirects) == 20
+
+
 def test_values(mock_session):
     for url in [MOCKED_URL, MOCKED_URL_JSON, MOCKED_URL_HTTPS]:
         mock_session.get(url)
@@ -200,7 +199,7 @@ def test_values__with_invalid_responses(check_expiry, expected_count, mock_sessi
     responses[1] = AttributeError
     responses[2] = CachedResponse(expires=YESTERDAY, url='test')
 
-    with patch.object(DbPickleDict, '__getitem__', side_effect=responses):
+    with patch.object(SQLitePickleDict, '__getitem__', side_effect=responses):
         values = mock_session.cache.values(check_expiry=check_expiry)
         assert len(list(values)) == expected_count
 
@@ -247,6 +246,17 @@ def test_filter_fn__retroactive(mock_session):
     assert not mock_session.cache.has_url(MOCKED_URL_JSON)
 
 
+# def test_key_fn(mock_session):
+#     def create_key(request, **kwargs):
+#         """Create a key based on only the request URL (without params)"""
+#         return request.url.split('?')[0]
+
+#     mock_session.cache.key_fn = create_key
+#     mock_session.get(MOCKED_URL)
+#     response = mock_session.get(MOCKED_URL, params={'k': 'v'})
+#     assert response.from_cache is True
+
+
 def test_hooks(mock_session):
     state = defaultdict(int)
     mock_session.get(MOCKED_URL)
@@ -328,12 +338,30 @@ def test_clear(mock_session):
     assert not mock_session.cache.has_url(MOCKED_URL_REDIRECT)
 
 
+def test_has_url(mock_session):
+    mock_session.get(MOCKED_URL)
+    assert mock_session.cache.has_url(MOCKED_URL)
+    assert not mock_session.cache.has_url(MOCKED_URL_REDIRECT)
+
+
+def test_has_url__request_args(mock_session):
+    mock_session.get(MOCKED_URL, params={'foo': 'bar'})
+    assert mock_session.cache.has_url(MOCKED_URL, params={'foo': 'bar'})
+    assert not mock_session.cache.has_url(MOCKED_URL)
+
+
 def test_delete_url(mock_session):
     mock_session.get(MOCKED_URL)
     mock_session.cache.delete_url(MOCKED_URL)
     assert not mock_session.cache.has_url(MOCKED_URL)
 
 
+def test_delete_url__request_args(mock_session):
+    mock_session.get(MOCKED_URL, params={'foo': 'bar'})
+    mock_session.cache.delete_url(MOCKED_URL, params={'foo': 'bar'})
+    assert not mock_session.cache.has_url(MOCKED_URL, params={'foo': 'bar'})
+
+
 def test_delete_url__nonexistent_response(mock_session):
     """Deleting a response that was either already deleted (or never added) should fail silently"""
     mock_session.cache.delete_url(MOCKED_URL)
@@ -374,8 +402,9 @@ def test_response_defaults(mock_session):
     response_1 = mock_session.get(MOCKED_URL)
     response_2 = mock_session.get(MOCKED_URL)
     response_3 = mock_session.get(MOCKED_URL)
+    cache_key = '71c046cdb0afaa62'
 
-    assert response_1.cache_key.startswith('4398752d')
+    assert response_1.cache_key == cache_key
     assert response_1.created_at is None
     assert response_1.expires is None
     assert response_1.from_cache is False
@@ -383,43 +412,49 @@ def test_response_defaults(mock_session):
 
     assert isinstance(response_2.created_at, datetime)
     assert isinstance(response_2.expires, datetime)
-    assert response_2.cache_key.startswith('4398752d')
+    assert response_2.cache_key == cache_key
     assert response_2.created_at == response_3.created_at
     assert response_2.expires == response_3.expires
     assert response_2.from_cache is response_3.from_cache is True
     assert response_2.is_expired is response_3.is_expired is False
 
 
-def test_include_get_headers(mock_session):
-    """With include_get_headers, requests with different headers should have different cache keys"""
-    mock_session.cache.include_get_headers = True
+def test_match_headers(mock_session):
+    """With match_headers, requests with different headers should have different cache keys"""
+    mock_session.cache.match_headers = True
     headers_list = [{'Accept': 'text/json'}, {'Accept': 'text/xml'}, {'Accept': 'custom'}, None]
     for headers in headers_list:
         assert mock_session.get(MOCKED_URL, headers=headers).from_cache is False
         assert mock_session.get(MOCKED_URL, headers=headers).from_cache is True
 
 
-def test_include_get_headers_normalize(mock_session):
-    """With include_get_headers, the same headers (in any order) should have the same cache key"""
-    mock_session.cache.include_get_headers = True
+def test_match_headers_normalize(mock_session):
+    """With match_headers, the same headers (in any order) should have the same cache key"""
+    mock_session.cache.match_headers = True
     headers = {'Accept': 'text/json', 'Custom': 'abc'}
     reversed_headers = {'Custom': 'abc', 'Accept': 'text/json'}
     assert mock_session.get(MOCKED_URL, headers=headers).from_cache is False
     assert mock_session.get(MOCKED_URL, headers=reversed_headers).from_cache is True
 
 
+def test_include_get_headers():
+    """include_get_headers is aliased to match_headers for backwards-compatibility"""
+    session = CachedSession(include_get_headers=True, backend='memory')
+    assert session.cache.match_headers is True
+
+
 @pytest.mark.parametrize('exception_cls', DESERIALIZE_ERRORS)
 def test_cache_error(exception_cls, mock_session):
     """If there is an error while fetching a cached response, a new one should be fetched"""
     mock_session.get(MOCKED_URL)
-    with patch.object(DbDict, '__getitem__', side_effect=exception_cls):
+    with patch.object(SQLiteDict, '__getitem__', side_effect=exception_cls):
         assert mock_session.get(MOCKED_URL).from_cache is False
 
 
 def test_expired_request_error(mock_session):
-    """Without old_data_on_error (default), if there is an error while re-fetching an expired
+    """Without stale_if_error (default), if there is an error while re-fetching an expired
     response, the request should be re-raised and the expired item deleted"""
-    mock_session.old_data_on_error = False
+    mock_session.stale_if_error = False
     mock_session.expire_after = 0.01
     mock_session.get(MOCKED_URL)
     time.sleep(0.01)
@@ -430,9 +465,9 @@ def test_expired_request_error(mock_session):
     assert len(mock_session.cache.responses) == 0
 
 
-def test_old_data_on_error__exception(mock_session):
-    """With old_data_on_error, expect to get old cache data if there is an exception during a request"""
-    mock_session.old_data_on_error = True
+def test_stale_if_error__exception(mock_session):
+    """With stale_if_error, expect to get old cache data if there is an exception during a request"""
+    mock_session.stale_if_error = True
     mock_session.expire_after = 0.2
 
     assert mock_session.get(MOCKED_URL).from_cache is False
@@ -443,9 +478,9 @@ def test_old_data_on_error__exception(mock_session):
         assert response.from_cache is True and response.is_expired is True
 
 
-def test_old_data_on_error__error_code(mock_session):
-    """With old_data_on_error, expect to get old cache data if a response has an error status code"""
-    mock_session.old_data_on_error = True
+def test_stale_if_error__error_code(mock_session):
+    """With stale_if_error, expect to get old cache data if a response has an error status code"""
+    mock_session.stale_if_error = True
     mock_session.expire_after = 0.2
     mock_session.allowable_codes = (200, 404)
 
@@ -456,6 +491,12 @@ def test_old_data_on_error__error_code(mock_session):
     assert response.from_cache is True and response.is_expired is True
 
 
+def test_old_data_on_error():
+    """stale_if_error is aliased to old_data_on_error for backwards-compatibility"""
+    session = CachedSession(old_data_on_error=True, backend='memory')
+    assert session.stale_if_error is True
+
+
 def test_cache_disabled(mock_session):
     mock_session.get(MOCKED_URL)
     with mock_session.cache_disabled():
@@ -487,7 +528,33 @@ def test_do_not_cache(mock_session):
     assert not mock_session.cache.has_url(MOCKED_URL_JSON)
 
 
-def test_url_whitelist(mock_session):
+@pytest.mark.parametrize(
+    'response_code, cache_hit, cache_expired, expected_from_cache',
+    [
+        # For 200 responses, never return stale cache data
+        (200, False, False, False),
+        (200, True, False, True),
+        (200, True, True, False),
+        # For 304 responses, return stale cache data
+        (304, False, False, False),
+        (304, True, False, True),
+        (304, True, True, True),
+    ],
+)
+def test_304_not_modified(response_code, cache_hit, cache_expired, expected_from_cache, mock_session):
+    url = f'{MOCKED_URL}/endpoint_2'
+    if cache_expired:
+        mock_session.expire_after = datetime.now() - timedelta(1)
+    if cache_hit:
+        mock_session.mock_adapter.register_uri('GET', url, status_code=200)
+        mock_session.get(url)
+    mock_session.mock_adapter.register_uri('GET', url, status_code=response_code)
+
+    response = mock_session.get(url)
+    assert response.from_cache is expected_from_cache
+
+
+def test_url_allowlist(mock_session):
     """If the default is 0, only URLs matching patterns in urls_expire_after should be cached"""
     mock_session.urls_expire_after = {
         MOCKED_URL_JSON: 60,
@@ -527,11 +594,11 @@ def test_remove_expired_responses__error(mock_session):
     mock_session.get(MOCKED_URL_JSON)
 
     def error_on_key(key):
-        if key == url_to_key(MOCKED_URL_JSON):
+        if key == create_key(method='GET', url=MOCKED_URL_JSON):
             raise PickleError
         return mock_session.get(MOCKED_URL_JSON)
 
-    with patch.object(DbPickleDict, '__getitem__', side_effect=error_on_key):
+    with patch.object(SQLitePickleDict, '__getitem__', side_effect=error_on_key):
         mock_session.remove_expired_responses()
     assert len(mock_session.cache.responses) == 1
     assert mock_session.get(MOCKED_URL).from_cache is True
@@ -609,7 +676,7 @@ def test_unpickle_errors(mock_session):
     """If there is an error during deserialization, the request should be made again"""
     assert mock_session.get(MOCKED_URL_JSON).from_cache is False
 
-    with patch.object(DbPickleDict, '__getitem__', side_effect=PickleError):
+    with patch.object(SQLitePickleDict, '__getitem__', side_effect=PickleError):
         resp = mock_session.get(MOCKED_URL_JSON)
         assert resp.from_cache is False
         assert resp.json()['message'] == 'mock json response'
